/*
 * Copyright (c) 2017      Intel Corporation. All rights reserved.
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */

#ifndef DATA_HOLDER_H
#define DATA_HOLDER_H

#include <map>
#include <string>
#include <vector>
#include <stdint.h>

typedef uint8_t dataByte;

class dataHolder {
    public:
        dataHolder(void) {};

        virtual ~dataHolder(void) {};

        template<typename T> inline dataHolder(const T& value) {
            size_t s = sizeof(T);
            dataByte* ptr = (dataByte*) &value;
            storedData.insert(storedData.begin(), ptr, ptr + s);
            dataTypeName = std::string(typeid(T).name());
        }

        template<typename T> bool is() {
            return (0 == dataTypeName.compare(typeid(T).name()));
        }

        template<typename T> inline T* ptr() {
            return reinterpret_cast<T*>(storedData.data());
        };

        template<typename T> inline T getValue() {
            return *((T*) storedData.data());
        }

        inline size_t size() {
            return storedData.size()*sizeof(dataByte);
        };

        inline const std::string getDataTypeName() const {
            return dataTypeName;
        };

    protected:
        std::string dataTypeName;
        std::vector<dataByte> storedData;
};

template <> inline std::string dataHolder::getValue() {
    char* s = (char*) storedData.data();
    return std::string(s);
}

template <> inline dataHolder::dataHolder(const std::string& value) {
    size_t s = value.length() + 1; // +1 because of NULL terminator
    dataByte* ptr = (dataByte*) value.c_str();
    storedData.insert(storedData.begin(), ptr, ptr + s);
    dataTypeName = std::string(typeid(std::string).name());
}

typedef std::map<std::string, dataHolder> dataHolderMap;

#endif
